import java.io.*;
 
/**
 * An output stream which encodes a file into base64 format.  
 *
 * @version     1.1, 12 Jan 1997
 * @author      Glenn Vanderburg
 */

public 
class Base64OutputStream extends FilterOutputStream 
{
    byte buf[] = new byte[3];
    int buffered = 0;
    boolean textfile;
    int chunksdone = 0;
    boolean finished = false;

    /* Base64 encoding bytes */
/* BUGFIX: Instead of Y and y there was U and u */
    static byte b[] = {
        'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', // 0x00 -- 0x07
        'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', // 0x08 -- 0x0F
        'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', // 0x10 -- 0x17
        'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', // 0x18 -- 0x1F
        'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', // 0x20 -- 0x27
        'o', 'p', 'q', 'r', 's', 't', 'u', 'v', // 0x28 -- 0x2F
        'w', 'x', 'y', 'z', '0', '1', '2', '3', // 0x30 -- 0x37
        '4', '5', '6', '7', '8', '9', '+', '/', // 0x38 -- 0x3F
    };

    static byte PAD = '=';

    /**
     * Creates a new Base64OutputStream.
     * @param out       the output stream
     */
    public
    Base64OutputStream (OutputStream out) 
    {
        this(out, false);
    }

    /**
     * Creates a new Base64OutputStream, for a text file.
     * @param out       the output stream
     * @param textfile  true if the file is a text file.
     */
    public
    Base64OutputStream (OutputStream out, boolean textfile) 
    {
        super(out);
        this.textfile = textfile;
    }

    /**
     * Writes a byte. This method will block until the byte is
     * actually written.
     * @param b the byte
     * @exception IOException If an I/O error has occurred.
     */
    public synchronized void
    write (int b) throws IOException
    {
        if (textfile && b == '\n') {
            write('\r');
        }

        buf[buffered++] = (byte) b;
        if (buffered == buf.length) {
            empty_buffer();
        }
    }
    
    /**
     * Writes a subarray of bytes.
     * @param b	  the data to be written
     * @param off the start offset in the data
     * @param len the number of bytes that are written
     * @exception IOException If an I/O error has occurred.
     */
    public void
    write (byte b[], int off, int len) throws IOException
    {
	for (int i = 0 ; i < len ; i++) {
	    write(b[off + i]);
	}
    }

    /**
     * Closes the stream.
     * @exception IOException If an I/O error has occurred.
     */
    public synchronized void
    close () throws IOException
    {
        empty_buffer();
        super.close();
    }

    /**
     * Encode all buffered bytes and write the encoded
     * bytes to the output stream.
     * @exception IOException If an I/O error has occurred.
     */
    protected void 
    empty_buffer () throws IOException
    {
        if (buffered == 0) {  // Just for safety ...
            return;
        }

        if (finished) {  // Sorry, the padding's already on ...
            throw new BadFormatException();
        }

        for (int i=buffered; i < buf.length; i++) {
            buf[i] = 0;
        }

        // It's not so easy to do this with nice math as it is with
        // the decoding operation.  So,
/* BUGFIX: Shift left only 4 not 6 and &0x3f/&0x30/&0x0f. This is needed, as byte is enlarged to int
before shifting WITH sign extension, so shifting yields a large int on negative numbers */
        if (buffered > 0)
        {
            out.write(b[(buf[0]>>2)&0x3f]);
            out.write(b[((buf[0]<<4)&0x30) | ((buf[1]>>4)&0x0f)]);
        }
/* BUGFIX: Shift left only 2 not 4 and &0x3c/0x03, see above */
        if (buffered > 1)
        {
            out.write(b[((buf[1]<<2)&0x3c) | ((buf[2]>>6)&0x03)]);
        }
        else
        {
            out.write(PAD);
        }

        if (buffered > 2)
        {
            out.write(b[buf[2]&0x3f]);
            chunksdone++;
        }
        else
        {
            out.write(PAD);
            finished = true;
        }

        buffered = 0;
        if ((chunksdone == 18) || finished) {
            out.write('\n');
            chunksdone = 0;
        }
    }
}
