import java.io.*;

/**
Kopiert eine Datei. Je nach Kommandozeilen-Parametern mit oder ohne Bufferung beim Lesen bzw. Schreiben.
RESULTS:
src.bin: 1.056.528 Bytes
Ohne Buffern: 153 s
InputBuffer: 120 s
OutputBuffer: 101 s
In-&OutputBuffer: 40 s
@author Dipl.-Ing. Michael Sonntag
@version 1.0
*/
public class BufferTest
{

static public void main(String args[])
{
    int argIndex=2;
    boolean bufferInput=false;  // Buffer input?
    boolean bufferOutput=false; // Buffer output?
    // Read parameters
    if(args.length<2)
    {
        System.out.println("Too few parameters.\nUsage: java BufferTest sourcefile destinationfile [-i] [-o]");
        System.exit(0);
    }
    while(argIndex<args.length)
    {
        if(args[argIndex].equalsIgnoreCase("-i"))
            bufferInput=true;
        else if(args[argIndex].equalsIgnoreCase("-o"))
            bufferOutput=true;
        else
        {
            System.out.println("Unknown argument: "+args[argIndex]+".\nUsage: java BufferTest sourcefile destinationfile [-i] [-o]");
            System.exit(0);
        }
        argIndex++;
    }
    File src=new File(args[0]);
    if(!src.exists() || !src.isFile() || !src.canRead())
    {
        System.out.println("Source file doesn't exist or cannot be read.\nUsage: java BufferTest sourcefile destinationfile [-i] [-o]");
        System.exit(0);
    }
    File dest=new File(args[1]);
    if(dest.exists())
    {   // File exists -> check if it can be written to
        if(!dest.isFile() || !dest.canWrite())
        {
            System.out.println("Destination file cannot be written.\nUsage: java BufferTest sourcefile destinationfile [-i] [-o]");
            System.exit(0);
        }
    }
    else
    {   // Check if parent directory can be written to
        String parent=dest.getParent();
        // If no parent, look in current directory
        if(parent==null)
            parent=System.getProperty("user.dir");
        File dir=new File(parent);
        if(!dir.exists() || !dir.isDirectory() || !dir.canWrite())
        {
            System.out.println("Cannot write to destination directory.\nUsage: java BufferTest sourcefile destinationfile [-i] [-o]");
            System.exit(0);
        }
    }
    // Print status
    if(bufferInput)
        System.out.println("Using input buffer...");
    if(bufferOutput)
        System.out.println("Using output buffer...");
    InputStream in=null;
    OutputStream out=null;
    try
    {
        // Open streams (with or without buffer)
        InputStream fSrc=new FileInputStream(src);
        OutputStream fDest=new FileOutputStream(dest);
        if(bufferInput)
            in=new BufferedInputStream(fSrc);
        else
            in=fSrc;
        if(bufferOutput)
            out=new BufferedOutputStream(fDest);
        else
            out=fDest;
        int buffer;
        // Copy single byte only till EOF
        long start=System.currentTimeMillis();  // Start timming
        while((buffer=in.read())!=-1)
            out.write(buffer);
        long end=System.currentTimeMillis();   // End timing
        System.out.println("Elapsed time in seconds: "+(end-start)/1000);
    }
    catch(IOException e)
    {
        System.out.println("Error during copying: "+e.toString());
    }
    finally
    {   // Close in any case
        try
        {
            if(in!=null)
                in.close();
            if(out!=null)
                out.close();
        }
        catch(IOException e)
        {   // Nothing can be done here
        }
    }
    
}

}

