import java.io.*;

/**
GrepReader: Filtert aus Textzeilen jene heraus, die eine bestimmte Zeichenkette enthalten
Das Filtern funktioniert nur bei readLine, die anderen Methoden (read, read([]),...)
funktionieren wie bisher
@author Dipl.-Ing. Michael Sonntag
@version 1.0
*/
public class GrepReader extends BufferedReader
{
    String muster; // String, nach dem gesucht wird
  
    public GrepReader(Reader in, String muster) 
    {
        super(in);
        this.muster=muster;
        // Warum nicht this.patter=new String(pattern); ?!?!!!!
    }

    public String readLine() throws IOException
    {
        String line;
        // Zeile einlesen, bis entweder Dateiende oder Zeile mit Muster gefunden
        do
        {
            line=super.readLine();
        } while (line!=null && line.indexOf(muster)==-1);
        return line;
    }
    
    
    
    public static void main(String[] args)
    {
        // Parameter prfen
        if(args.length!=2)
        {
            System.err.println("Benutzung: java GrepReader Dateiname Muster");
            System.exit(1);
        }
        File datei=new File(args[0]);
        if(!datei.exists() || !datei.canRead() || !datei.isFile())
        {
            System.err.println("Benutzung: java GrepReader Dateiname");
            System.exit(1);
        }
        // Neuen GrepReader erzeugen und entsprechende Reader bergeben
        try
        {
            GrepReader grep=new GrepReader(new FileReader(datei),args[1]);
            String line;
            while((line=grep.readLine())!=null)
                System.out.println(line);
            grep.close();
        }
        catch(FileNotFoundException e)
        {   // Sollte eigentlich nicht auftreten: AUSSER Datei ist bereits von anderem Programm geffnet
            System.err.println("Fehler beim ffnen der Datei: "+e.toString());
            System.exit(1);
        }
        catch(IOException e)
        {
            System.err.println("Fehler beim Lesen der Datei: "+e.toString());
            System.exit(1);
        }
    }
}