import java.io.*;

/**
Hexdump - Gibt einen Hexdump einer Datei aus: Jede Zeile 16 Bytes in Hex und rechts davon in ASCII.
Nicht darstellbare Zeichen werden durch einen Punkt ersetzt.
@author Dipl.-Ing. Michael Sonntag
@version 1.0
*/
public class Hexdump
{
    
private final static int BREITE=16;

static public void main(String args[])
{
    File datei; // Die Datei die ausgegeben werden soll
    int inByte; // Gerade gelesenes Byte
    int pos=0;  // Position in der Zeile
    long count=0;   // Bisher ausgegebene Bytes   
    StringBuffer ascii=new StringBuffer(BREITE+1); // Buffer fr die ASCII-Reprsentation
    // +1 um sicherzugehen, da der Buffer nicht vergert wird
    if(args.length!=1)
    {
        System.err.println("Benutzung: java Hexdump Dateiname");
        System.exit(1);
    }
    // Datei prfen
    datei=new File(args[0]);
    if(!datei.exists() || !datei.canRead() || !datei.isFile())
    {
        System.err.println("Benutzung: java Hexdump Dateiname");
        System.exit(1);
    }
    try
    {
        // Inputstream ffnen, hier byteweise lesen
        BufferedInputStream in=new BufferedInputStream(new FileInputStream(datei));
        System.out.println("Hexdump of "+datei.getCanonicalPath()+":\n");
        while((inByte=in.read())>=0)
        {
            if(pos==0)
            {   // Zeilenbeginn: Offset bis hierher ausgeben
                String out=Long.toHexString(count).toUpperCase();
                for(int i=0;i<6-out.length();i++)
                    System.out.print("0");
                System.out.print(out+": ");
            }
            // toHexString liefert keine fhrenden Nullen, daher bei einstellig hndisch erweitern
            if(inByte<16)
                System.out.print("0");
            // Bytewert in Hex ausgeben
            System.out.print(Long.toHexString(inByte).toUpperCase());
            // Weiterzhlen
            pos++;
            count++;
            // Ascii-Reprsentation anhngen ('.' wenn nicht-druckbar)
            if(inByte>=32 && inByte<=127)
                ascii.append((char)inByte);
            else
                ascii.append(".");
            if(pos==BREITE)
            {   // Ende einer Zeile, Trennung und Ascii-Reprsentation ausgeben
                System.out.print("   ");
                System.out.println(ascii.toString());
                ascii=new StringBuffer(BREITE+1);
                pos=0;
            }
        }
    }
    catch(FileNotFoundException e)
    {   // Sollte eigentlich nicht auftreten: AUSSER Datei ist bereits von anderem Programm geffnet
        System.err.println("Fehler beim ffnen der Datei: "+e.toString());
        System.exit(1);
    }
    catch(IOException e)
    {
        System.err.println("Fehler beim Lesen der Datei: "+e.toString());
        System.exit(1);
    }
    // Rest der letzten Zeile ausgeben (sofern vorhanden)
    if(pos>0)
    {
        while(pos<BREITE)
        {
            System.out.print("  ");
            pos++;
        }
        System.out.print("   ");
        System.out.println(ascii.toString());
    }    
}

}

